// Libraries
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <FastLED.h>
#define NUM_LEDS 8
#define DATA_PIN 4
CRGB leds[NUM_LEDS];
int color = 20;
int dim = 80;
int modus = 1;
int confiG = 80;

// WiFi network
const char* ssid     = "your-ssid";
const char* password = "your-password";

ESP8266WebServer server ( 80 );



char htmlResponse[3000];

void handleRoot() {

  snprintf ( htmlResponse, 3000,
             "<!DOCTYPE html>\
<html lang=\"en\">\
  <head>\
    <meta charset=\"utf-8\">\
    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">\
  </head>\
  <body>\
          <h1><font color='red'>mood light</font></h1>\
          <input type='range' name='color_slider' id='color_slider' size=2 min='5' max='100' value='50'> effect speed<br> \
          <input type='range' name='dim_slider' id='dim_slider' size=2 min='1' max='254' value='125'> brightness<br> \
          <select id='date_ss'>\
            <option value='5'>pause</option>\
            <option value='6'>off</option>\
            <option value='1'>fade</option>\
            <option value='2'>rainbow</option>\
            <option value='3'>droplets</option>\
            <option value='4'>cyclon</option>\
            <option value='7'>light blue-pink</option>\
            <option value='8'>blue-purple</option>\
            <option value='9'>green-blue</option>\
            <option value='10'>orange-red</option>\
          </select> modus\
          <div>\
          <br><button id=\"save_button\" size='3'>Save</button>\
          </div>\
    <script src=\"https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js\"></script>\
    <script>\
      var cs;\
      var ds;\
      var ss;\
      $('#save_button').click(function(e){\
        e.preventDefault();\
        cs = $('#color_slider').val();\
        ds = $('#dim_slider').val();\
        ss = $('#date_ss').val();\
        $.get('/save?cs=' + cs + '&ds=' + ds + '&ss=' + ss, function(data){\
          console.log(data);\
        });\
      });\
    </script>\
  </body>\
</html>");

  server.send ( 200, "text/html", htmlResponse );

}


void handleSave() {
  if (server.arg("cs") != "") {
    Serial.println("color: " + server.arg("cs"));
    //String b = server.arg("cs");
    color = 105 - server.arg("cs").toInt();
  }

  if (server.arg("ds") != "") {
    Serial.println("brightness: " + server.arg("ds"));
    dim = server.arg("ds").toInt();
  }

  if (server.arg("ss") != "") {
    Serial.println("moduds: " + server.arg("ss"));
    modus = server.arg("ss").toInt();
  }
}


void setup() {
  LEDS.addLeds<WS2812, DATA_PIN, RGB>(leds, NUM_LEDS);
  // Start serial
  Serial.begin(115200);
  delay(10);

  // Connecting to a WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    for (int i = 0; i < NUM_LEDS; i++) {
      leds[i] = CHSV(0, 255, 100);
    }
    FastLED.show();
    delay(250);
    for (int i = 0; i < NUM_LEDS; i++) {
      leds[i] = CHSV(0, 0, 0);
    }
    FastLED.show();
    delay(250);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  server.on ( "/", handleRoot );
  server.on ("/save", handleSave);

  server.begin();
  Serial.println ( "HTTP server started" );
}

void loop() {
  server.handleClient();
  switch (modus) {
    case 1:
      modus1();
      break;
    case 2:
      modus2();
      break;
    case 3:
      modus3();
      break;
    case 4:
      modus4();
      break;
    case 5:
      modus5();
      break;
    case 6:
      modus6();
      break;
    case 7:
      modus7();
      break;
    case 8:
      modus8();
      break;
    case 9:
      modus9();
      break;
    case 10:
      modus10();
      break;
    default:
      modus1();
      break;
  }
}

void modus1() {
  static int counter = 0;
  counter++;
  if (counter > 255) {
    counter = 0;
  }
  delay(map(color, 5, 100, 5, 500));
  for (int i = 0; i < NUM_LEDS; i++) {
    leds[i] = CHSV(counter, 255, dim);
  }
  FastLED.show();
}

void modus2() {
  static int counter = 0;
  delay(map(color, 5, 100, 5, 500));
  if (counter > 255) {
    counter = 0;
  }
  counter++;
  for (int i = 0; i <= NUM_LEDS; i++) {
    leds[i] = CHSV(counter + i, 255, dim);
  }
  FastLED.show();
}

void modus3() {
  static float counter = 0;
  static float C;
  int nummerdrup = map(color, 0, 255, 0, 10);
  counter = counter + 0.001;
  for (int i = 0; i <= nummerdrup; i++) {
    int pos = random16(NUM_LEDS);
    C = sin(counter) * 200;
    leds[pos] += CHSV(C + random8(30), 255, dim);
  }
  delay(30);
  fadeall();
  if (counter > 10000) {
    counter = 0;
  }
  FastLED.show();
}

void modus4() {
  static uint8_t hue = 0;
  static int counter = 0;
  static boolean dir = true;
  if (dir == true) {
    counter++;
  } else if (dir == false) {
    counter--;
  }
  if (counter > NUM_LEDS) {
    dir = false;
  }
  if (counter < 0) {
    dir = true;
  }
  leds[counter] = CHSV(hue++, 255, dim);
  FastLED.show();
  fadeall();
  delay(map(color, 5, 100, 5, 50));
}

void modus5() {
  delay(20);
}

void modus6() {
  for (int i = 0; i <= NUM_LEDS; i++) {
    leds[i] = CHSV(i, 0, 0);
  }
  FastLED.show();
  delay(20);
}

void modus7() {
  // 160 - 255
  // 95
  int minimum = 160;
  int maximum = 240;
  int avg = (minimum + maximum) / 2;
  int difference = maximum - minimum;
  static float counter;

  float si = map(confiG, 5, 100, 1, 100);
  counter += si / 1000;

  Serial.println(difference / 2 * sin(counter) + avg);

  for (int i = 0; i <= NUM_LEDS; i++) {
    float Delay = i * si / 1000;
    float colorgradient = difference / 2 * sin(counter + Delay) + avg;
    leds[i] = CHSV(colorgradient , 255, dim);
  }

  FastLED.show();
  delay(map(color, 5, 100, 5, 100));

}

void modus8() {
  // 160 - 255
  // 95
  int minimum = 170;
  int maximum = 200;
  int avg = (minimum + maximum) / 2;
  int difference = maximum - minimum;
  static float counter;

  float si = map(confiG, 5, 100, 1, 100);
  counter += si / 1000;

  Serial.println(difference / 2 * sin(counter) + avg);

  for (int i = 0; i <= NUM_LEDS; i++) {
    float Delay = i * si / 1000;
    float colorgradient = difference / 2 * sin(counter + Delay) + avg;
    leds[i] = CHSV(colorgradient , 255, dim);
  }

  FastLED.show();
  delay(map(color, 5, 100, 5, 100));

}

void modus9() {
  // 160 - 255
  // 95
  int minimum = 96;
  int maximum = 180;
  int avg = (minimum + maximum) / 2;
  int difference = maximum - minimum;
  static float counter;

  float si = map(confiG, 5, 100, 1, 100);
  counter += si / 1000;

  Serial.println(difference / 2 * sin(counter) + avg);

  for (int i = 0; i <= NUM_LEDS; i++) {
    float Delay = i * si / 1000;
    float colorgradient = difference / 2 * sin(counter + Delay) + avg;
    leds[i] = CHSV(colorgradient , 255, dim);
  }

  FastLED.show();
  delay(map(color, 5, 100, 5, 100));

}


void modus10() {
  // 160 - 255
  // 95
  int minimum = 0;
  int maximum = 32;
  int avg = (minimum + maximum) / 2;
  int difference = maximum - minimum;
  static float counter;

  float si = map(confiG, 5, 100, 1, 100);
  counter += si / 1000;

  Serial.println(difference / 2 * sin(counter) + avg);

  for (int i = 0; i <= NUM_LEDS; i++) {
    float Delay = i * si / 1000;
    float colorgradient = difference / 2 * sin(counter + Delay) + avg;
    leds[i] = CHSV(colorgradient , 255, dim);
  }

  FastLED.show();
  delay(map(color, 5, 100, 5, 100));

}


void fadeall() {
  for (int i = 0; i < NUM_LEDS; i++) {
    leds[i].nscale8(240);
  }
}
